# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-03-30 12:52
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('collector', '0007_loggerindex_num_processes'),
    ]

    operations = [
        migrations.AddField(
            model_name='loggercolumn',
            name='indexed',
            field=models.BooleanField(default=False),
        ),
    ]



# coding: utf-8

# $Id: $
import socket
import time
import ujson as json
import sys
from datetime import datetime
from collections import defaultdict
from random import randint
from logging import getLogger
from threading import Thread

import signal

from django.core.signals import request_started, request_finished
from django.db import connections, DatabaseError, ProgrammingError
from django.utils import six
import redis
from librabbitmq import Connection
from pyrabbit.api import Client

# noinspection PyUnresolvedReferences
from six.moves import queue

from alco.collector.defaults import ALCO_SETTINGS
from alco.collector import keys


class Collector(object):

    def __init__(self, index):
        self.index = index
        self.cancelled = False
        self.transport = self.protocol = None
        self.messages = []
        self.block_size = 1000
        self.exchange = "logstash"
        self.current_date = None
        self.logger = getLogger('alco.collector.%s' % self.index.name)
        self.amqp = self.redis = self.conn = self.vhost = self.rabbit = None
        self.insert_thread = None
        self.query_queue = queue.Queue()
        self.result_queue = queue.Queue()
        self.result_queue.put(None)
        self.query = self.values_stub = None
        self.existing = self.included = self.indexed = self.filtered = None

    def cancel(self):
        self.cancelled = True

    def inserter_loop(self):
        self.conn = connections[ALCO_SETTINGS['SPHINX_DATABASE_NAME']]
        while not self.cancelled:
            try:
                query, args = self.query_queue.get(block=True, timeout=1)
            except queue.Empty:
                continue
            result = self.insert_data(query, args)
            self.result_queue.put(result)

    def connect(self):
        connections['default'].close()
        rabbitmq = ALCO_SETTINGS['RABBITMQ']
        self.amqp = Connection(**rabbitmq)
        self.redis = redis.Redis(**ALCO_SETTINGS['REDIS'])
        self.insert_thread = Thread(target=self.inserter_loop)
        self.insert_thread.start()
        hostname = '%s:%s' % (rabbitmq['host'],
                              ALCO_SETTINGS['RABBITMQ_API_PORT'])
        self.rabbit = Client(hostname, rabbitmq['userid'], rabbitmq['password'])
        self.vhost = rabbitmq['virtual_host']

    # noinspection PyUnusedLocal
    def process_sigint(self, signum, frame):
        self.logger.info("Got signal %s" % signum)
        self.cancel()
        self.logger.info("Futures cancelled, wait for thread")
        self.insert_thread.join()
        self.logger.info("Thread done")

    def __call__(self):
        signal.signal(signal.SIGINT, self.process_sigint)
        signal.signal(signal.SIGTERM, self.process_sigint)

        try:
            self.logger.debug("Connecting to RabbitMQ")
            self.connect()
            self.declare_queue()
            self.cleanup_bindings()
            channel = self.amqp.channel()
            channel.basic_qos(0, 1000, False)
            channel.basic_consume(self.index.queue_name,
                                  callback=self.process_message, no_ack=True)
            start = time.time()
            self.logger.debug("Start processing messages")
            while not self.cancelled:
                try:
                    self.amqp.drain_events(timeout=1)
                except (socket.timeout, OSError):
                    pass
                if time.time() - start > 1:
                    self.push_messages()
                    start = time.time()
        except KeyboardInterrupt:
            self.logger.warning("Got SIGINT, exit(0)")
        finally:
            self.amqp.close()
            sys.exit(0)

    def process_message(self, msg):
        data = json.loads(six.binary_type(msg.body))
        ts = data.pop('@timestamp')
        data.pop("@version")
        msg = data.pop('message')
        seq = data.pop('seq', 0)
        dt = datetime.strptime(ts, "%Y-%m-%dT%H:%M:%S.%fZ")
        result = {
            'ts': time.mktime(dt.timetuple()),
            'ms': dt.microsecond,
            'seq': seq,
            'message': msg,
            'data': data
        }
        self.messages.append(result)
        d = dt.date()
        if not self.current_date:
            self.current_date = d
        if d != self.current_date:
            self.current_date = d
            self.push_messages()
        if len(self.messages) >= self.block_size:
            self.push_messages()

    def declare_queue(self):
        channel = self.amqp.channel()
        """:type channel: amqp.channel.Channel"""
        durable = self.index.durable
        channel.exchange_declare(exchange=self.exchange, type='topic',
                                 durable=durable, auto_delete=False)
        channel.queue_declare(self.index.queue_name, durable=durable,
                              auto_delete=False)
        for rk in self.get_routing_keys():
            channel.queue_bind(self.index.queue_name, exchange=self.exchange,
                               routing_key=rk)

    def get_routing_keys(self):
        return map(lambda x: x.strip(), self.index.routing_key.split(','))

    def cleanup_bindings(self):
        self.logger.debug("Checking bindings")
        queue = self.index.queue_name
        exchange = self.exchange
        bindings = self.rabbit.get_queue_bindings(self.vhost, queue)
        bindings = [b for b in bindings if b['source'] == exchange]
        allowed = self.get_routing_keys()
        q = six.moves.urllib.parse.quote
        for b in bindings:
            rk = b['routing_key']
            if rk in allowed:
                continue
            self.logger.debug("Unbind %s with RK=%s" % (queue, rk))
            self.rabbit.delete_binding(self.vhost, exchange, q(queue), q(rk))

    def push_messages(self):
        try:
            request_started.send(None, environ=None)
            self._push_messages()
        except Exception as e:
            self.logger.exception(e)
            raise
        finally:
            request_finished.send(None)

    def _push_messages(self):
        messages, self.messages = self.messages, []
        if not messages:
            return
        message_count = len(messages)
        self.logger.info("Saving %s events" % message_count)
        columns = defaultdict(set)
        suffix = self.current_date.strftime("%Y%m%d")
        name = "%s_%s" % (self.index.name, suffix)
        args = []
        self.load_index_columns()
        self.prepare_query(name)

        pkeys = self.get_primary_keys(messages)
        seen = set()

        for pk, data in zip(pkeys, messages):
            # saving seen columns to LoggerColumn model, collecting unique
            # values for caching in redis

            js = data['data']
            self.process_js_columns(js, columns, self.included, seen)
            js_str = json.dumps(js)
            values = tuple(js.get(c) or '' for c in self.indexed)
            args.extend((pk, js_str, data['message']) + values)

        query = self.query + ','.join([self.values_stub] * message_count)

        self.save_column_values(columns)

        self.save_new_columns(seen)
        if self.result_queue.empty():
            self.logger.debug("Insert still running, waiting")
            while not self.cancelled:
                try:
                    self.result_queue.get(block=True, timeout=1)
                except queue.Empty:
                    continue

        self.query_queue.put((query, args))

    def insert_data(self, query, args):
        self.logger.debug("Inserting logs to searchd")
        result = None
        for _ in 1, 2, 3:
            try:
                c = self.conn.cursor()
                result = c.execute(query, args)
                self.logger.debug("%s rows inserted" % c.rowcount)
                c.close()
            except ProgrammingError:
                self.logger.exception(
                    "Can't insert values to index: %s" % query)
            except DatabaseError as e:
                self.logger.exception("Can't insert values to index: %s" % e)
            else:
                break
        return result

    def save_new_columns(self, seen):
        self.logger.debug("Check for new columns")
        for column in seen - set(self.existing):
            self.logger.debug("Register column %s" % column)
            self.index.loggercolumn_set.create(name=column)

    def save_column_values(self, columns):
        self.logger.debug("Saving values for filtered columns")
        ts = time.time()
        for column in self.filtered:
            values = columns.get(column)
            if not values:
                continue
            key = keys.KEY_COLUMN_VALUES.format(index=self.index.name,
                                                column=column)
            values = {v: ts for v in values}
            self.redis.zadd(key, **values)

    def prepare_query(self, name):
        if self.indexed:
            self.query = "REPLACE INTO %s (id, js, logline, %s) VALUES " % (
                name, ', '.join(self.indexed))
        else:
            self.query = "REPLACE INTO %s (id, js, logline) VALUES " % name

        sql_col_count = len(self.indexed) + 3  # + jd, js, logline
        self.values_stub = "(%s)" % ", ".join(["%s"] * sql_col_count)

    def load_index_columns(self):
        # all defined columns
        all_columns = list(self.index.loggercolumn_set.all())
        included_columns = [c for c in all_columns if not c.excluded]
        filtered_columns = [c for c in included_columns if c.filtered]
        indexed_columns = [c for c in included_columns if c.indexed]
        self.existing = [c.name for c in all_columns]
        self.included = [c.name for c in included_columns]
        self.filtered = [c.name for c in filtered_columns]
        self.indexed = [c.name for c in indexed_columns]

    @staticmethod
    def process_js_columns(js, columns, included, seen):
        for key, value in list(js.items()):
            if key in ('pk', 'id', 'ts', 'ms', 'seq'):
                # reserved by Django and ALCO
                js.pop(key)
                continue
            # save seen columns set
            if key not in seen:
                seen.add(key)
            if key not in included:
                # discard fields excluded from indexing
                js.pop(key)
                continue
            # save column values set
            if type(value) not in (bool, int, float, six.text_type):
                continue
            if value not in columns[key]:
                columns[key].add(value)

    def get_primary_keys(self, messages):
        """ Generate PK sequence for a list of messages."""
        pkeys = []
        pk = None
        for msg in messages:
            # pk is [timestamp][microseconds][randint] in 10based integer
            pk = int((msg['ts'] * 10**6 + msg['ms']) * 1000) + randint(0, 1000)
            pkeys.append(pk)
        self.logger.debug("first pk is %s" % pk)
        return pkeys



# coding: utf-8

# $Id: $
import os
import signal
from logging import getLogger
from multiprocessing import Process, Queue
from time import sleep

from daemon import DaemonContext
from daemon.pidfile import TimeoutPIDLockFile
from django.core.management import BaseCommand
from logutils.queue import QueueListener

from alco.collector.collector import Collector
from alco.collector.models import LoggerIndex

logger = getLogger('alco.collector')


def file_handles(logger):
    """ Get a list of filehandle numbers from logger
        to be handed to DaemonContext.files_preserve
    """
    handles = []
    for handler in logger.handlers:
        try:
            handles.append(handler.stream.fileno())
        except AttributeError:
            pass
    if logger.parent:
        handles += file_handles(logger.parent)
    return handles


class Command(BaseCommand):
    def __init__(self, stdout=None, stderr=None, no_color=False):
        super(Command, self).__init__(stdout=stdout, stderr=stderr,
                                      no_color=no_color)
        self.processes = {}
        self.started = False
        self.logging_queue = self.logging_listener = None

    def add_arguments(self, parser):
        parser.add_argument(
            '--no-daemon', action='store_false', dest='daemon',
            default=True, help="Don't daemonize process")
        parser.add_argument(
            '--pidfile', action='store', dest='pidfile',
            default="collector.pid", help="pidfile location")
        parser.add_argument(
            '--index', action='store', dest='index',
            default=None, help='index id'
        )

    def handle(self, *args, **options):
        if options['index']:
            index = LoggerIndex.objects.get(id=int(options['index']))
            c = Collector(index)
            c()
            return

        if not options['daemon']:
            return self.start_worker_pool()

        path = os.path.join(os.getcwd(), options['pidfile'])
        pidfile = TimeoutPIDLockFile(path)
        log_files = file_handles(logger)
        context = DaemonContext(pidfile=pidfile, files_preserve=log_files)

        with context:
            logger.info("daemonized")
            self.start_worker_pool()

    def start_worker_pool(self):
        self.start_mp_logging()
        indices = list(LoggerIndex.objects.all())
        logger.info("Starting worker pool with %s indexers" % len(indices))
        n = 0
        self.started = True
        for index in indices:
            for i in range(index.num_processes):
                c = Collector(index)
                p = Process(target=c)
                p.start()
                logger.info("Indexer for %s#%s started pid=%s" %
                            (index.name, i, p.pid))
                self.processes[n] = (p, index)
                n += 1
        signal.signal(signal.SIGINT, self.handle_sigint)

        while len(self.processes) > 0:
            if not self.started:
                logger.info("Stopping indexers")
                for n in self.processes:
                    (p, index) = self.processes[n]
                    os.kill(p.pid, signal.SIGTERM)
            sleep(0.5)
            for n in list(self.processes):
                (p, index) = self.processes[n]
                if p.exitcode is None:
                    if not p.is_alive() and self.started:
                        logger.debug("Indexer with pid %s not finished "
                                     "and not running" % p.pid)
                        # Not finished and not running
                        os.kill(p.pid, signal.SIGKILL)
                        c = Collector(index)
                        p = Process(target=c)
                        p.start()
                        logger.warning("Indexer for %s restarted pid=%s"
                                       % (index.name, p.pid))
                        self.processes[n] = (p, index)
                elif p.exitcode != 0 and self.started:
                    logger.warning('Process %s exited with an error '
                                   'or terminated' % p.pid)
                    c = Collector(index)
                    p = Process(target=c)
                    p.start()
                    self.processes[n] = (p, index)
                elif p.exitcode != 0:
                    logger.warning("Process %s exited with return code %s while"
                                   " terminating" % (p.pid, p.exitcode))
                    p.join()
                    del self.processes[n]
                else:
                    logger.debug('Process %s exited correctly' % p.pid)
                    p.join()
                    del self.processes[n]

        self.stop_mp_logging()

    def handle_sigint(self, sig_num, frame):
        logger.info("Got signal %s, stopping" % sig_num)
        self.started = False

    def start_mp_logging(self):
        logger.debug("Start listening child logs")
        self.logging_queue = Queue()
        handlers = logger.handlers
        self.logging_listener = QueueListener(self.logging_queue, *handlers)
        self.logging_listener.start()

    def stop_mp_logging(self):
        logger.debug("Stop listening child logs")
        self.logging_listener.stop()




# coding: utf-8

from collections import OrderedDict

from rest_framework.filters import BaseFilterBackend
from rest_framework.settings import api_settings
# noinspection PyPackageRequirements
from sphinxsearch.utils import sphinx_escape


class SphinxSearchFilter(BaseFilterBackend):
    # The URL query parameter used for the search.
    search_param = api_settings.SEARCH_PARAM

    @staticmethod
    def get_search_fields(request, view):
        search_fields = getattr(view, 'get_search_fields', None)
        if callable(search_fields):
            search_fields = search_fields(request)
        else:
            search_fields = getattr(view, 'search_fields', None)
        return search_fields

    def get_search_terms(self, request, escape=True):
        """
        Search terms are set by a ?search=... query parameter,
        and may be comma and/or whitespace delimited.
        """
        params = request.query_params.get(self.search_param, '')
        if not params or params == " ":
            return []
        if escape:
            return list(map(sphinx_escape, params.replace(',', ' ').split()))
        return params.replace(',', ' ').split()

    @staticmethod
    def get_match_terms(request, search_fields):
        result = {}
        for key in search_fields:
            value = request.query_params.get(key + '__in')
            if value is None:
                value = request.query_params.get(key)
                if value is None:
                    continue
            result[key] = value.split(',')
        return result

    def filter_queryset(self, request, queryset, view):
        search_fields = self.get_search_fields(request, view)
        search_terms = self.get_search_terms(request)
        if search_terms:
            queryset = queryset.match(logline='"%s"' % ' '.join(search_terms))

        match_terms = self.get_match_terms(request, search_fields)
        for field, values in match_terms.items():
            if len(values) == 1:
                queryset = queryset.filter(**{field: values[0]})
            else:
                values = '|'.join('"%s"' % sphinx_escape(v) for v in values)
                queryset = queryset.match(**{"%s" % field: values})

        select = dict(logline_snippet="SNIPPET(logline, %s, 'limit=1000000')")
        params = [' '.join(self.get_search_terms(request, escape=False))]
        return queryset.extra(select=select, select_params=params)


class JSONFieldFilter(BaseFilterBackend):

    @staticmethod
    def get_json_fields(request, view):
        json_fields = getattr(view, 'get_json_fields', None)
        if callable(json_fields):
            json_fields = json_fields(request)
        else:
            json_fields = getattr(view, 'json_fields', None)
        return json_fields

    def filter_queryset(self, request, queryset, view):
        json_fields = self.get_json_fields(request, view)
        if not json_fields:
            return queryset
        lookups = self.get_lookups(json_fields, request)
        all_values = set()
        for key, value in lookups.items():
            if not isinstance(value, (list, tuple)):
                value = [value]
            all_values.update(value)

        extra_lookups = OrderedDict()
        extra_params = []
        extra_where = []
        for lookup, values in lookups.items():
            field = lookup.split('__')[0]
            numeric = []
            for v in values:
                try:
                    numeric.append(int(v))
                except ValueError:
                    continue
            stub = ', '.join(['%s'] * len(values))
            extra_lookups['__%s_str' % lookup] = 'IN(js.%s, %s)' % (field, stub)
            extra_params.extend(values)
            if numeric:
                stub = ', '.join(['%s'] * len(numeric))
                extra_lookups['__%s_int' % lookup] = 'IN(js.%s, %s)' % (field,
                                                                        stub)
                extra_params.extend(numeric)
                extra_where.append('(__%s_str = 1 OR __%s_int = 1)'
                                   % (lookup, lookup))
            else:
                extra_where.append('__%s_str = 1' % lookup)
        queryset = queryset.extra(select=extra_lookups,
                                  select_params=extra_params,
                                  where=extra_where,
                                  params=[])
        return queryset

    @staticmethod
    def get_lookups(json_fields, request):
        lookups = {}
        for key in json_fields:
            value = request.query_params.get(key + '__in')
            if value is None:
                value = request.query_params.get(key)
                if value is None:
                    continue
            lookups[key + '__in'] = value.split(',')

        return lookups



# coding: utf-8

# $Id: $
from logging import getLogger

import redis
import time
from django.core.management import BaseCommand

from alco.collector import keys
from alco.collector.defaults import ALCO_SETTINGS
from alco.collector.models import LoggerIndex
from alco.grep.models import create_index_model


class Command(BaseCommand):
    """ Cleanup values for filtered columns from redis no more present in index.
    """
    logger = getLogger('alco.collector.cleanup_redis')
    redis = redis.Redis(**ALCO_SETTINGS['REDIS'])

    def handle(self, *args, **options):
        for index in LoggerIndex.objects.all():
            self.cleanup_index(index)

    def cleanup_column(self, Log, column, ts):
        self.logger.debug("Cleanup column %s@%s" %
                          (column.name, column.index.name))

        key = keys.KEY_COLUMN_VALUES.format(index=column.index.name,
                                            column=column.name)

        n = self.redis.zremrangebyscore(key, '-inf', ts)
        self.logger.debug("removed %s values" % n)

    def cleanup_index(self, index):
        self.logger.debug("Cleanup index %s" % index.name)
        Log = create_index_model(index)
        for column in index.loggercolumn_set.filter(filtered=True):
            first_day = index.index_dates[0]
            ts = time.mktime(first_day.timetuple())

            self.cleanup_column(Log, column, ts)



# coding: utf-8

# $Id: $

from rest_framework.pagination import CursorPagination

from alco.collector.defaults import ALCO_SETTINGS


class CursorLogPaginator(CursorPagination):
    page_size = ALCO_SETTINGS['LOG_PAGE_SIZE']
    ordering = ('id',)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-03-31 08:29
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('collector', '0008_loggercolumn_indexed'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='loggercolumn',
            unique_together=set([('index', 'name')]),
        ),
    ]



# coding: utf-8

# $Id: $
from copy import copy
from datetime import datetime, timedelta
import time

from django.http import HttpResponseRedirect
# noinspection PyUnresolvedReferences
from django.utils.six.moves.urllib import parse
from django.core.urlresolvers import reverse
from django.views.generic import DetailView, RedirectView, ListView
from django.views.generic.detail import SingleObjectMixin
import redis

from alco.collector import keys
from alco.collector.defaults import ALCO_SETTINGS
from alco.collector.models import LoggerIndex
from alco.grep.models import Shortcut

client = redis.Redis(**ALCO_SETTINGS['REDIS'])


class GrepView(DetailView):
    model = LoggerIndex
    slug_field = 'name'
    slug_url_kwarg = 'name'
    template_name = 'grep/log_view.html'

    def get(self, request, *args, **kwargs):
        if not request.GET.get('start_ts'):
            url = parse.urlparse(request.path)
            query_string = dict(parse.parse_qsl(request.META['QUERY_STRING']))
            start_ts = datetime.now().replace(microsecond=0) - timedelta(minutes=5)
            query_string['start_ts'] = start_ts.isoformat(sep=' ')
            query = parse.urlencode(query_string).replace('+', '%20')
            url = url._replace(query=query)
            return HttpResponseRedirect(parse.urlunparse(url))
        return super(GrepView, self).get(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        cd = super(GrepView, self).get_context_data(**kwargs)
        obj = self.get_object(self.get_queryset())
        filters = []
        columns = []
        ts = time.mktime(obj.index_dates[0].timetuple())
        for f in obj.loggercolumn_set.order_by('name'):
            if f.display:
                columns.append(f)
            if f.filtered:
                filters.append({
                    'title': f.name.title(),
                    'field': f.name,
                    'heading_id': 'heading-%s' % f.name,
                    'collapse_id': 'collapse-%s' % f.name,
                    'items': self.get_field_values(obj, f.name, ts)
                })
        cd['filters'] = filters
        cd['columns'] = columns
        cd['column_names'] = [c.name for c in columns]
        return cd

    @staticmethod
    def get_field_values(index, column, ts):
        key = keys.KEY_COLUMN_VALUES.format(index=index.name, column=column)
        return sorted(client.zrangebyscore(key, ts, '+inf'))


class ShortcutView(SingleObjectMixin, RedirectView):
    model = Shortcut
    permanent = False

    slug_url_kwarg = 'name'
    slug_field = 'name'

    def get_redirect_url(self, *args, **kwargs):
        obj = self.get_object()
        url = parse.urlparse(obj.url)
        query_params = copy(self.request.GET)
        lookup = self.kwargs.get('default_value')
        if lookup and obj.default_field:
            query_params[obj.default_field.name] = lookup

        query_params.update(**dict(parse.parse_qsl(url.query)))

        url = reverse('grep_view', kwargs={'name': obj.index.name})
        url = parse.urlparse(url)
        parts = list(url)
        parts[4] = parse.urlencode(query_params)
        return parse.urlunparse(parts)


class IndexListView(ListView):
    model = LoggerIndex

    template_name = 'grep/loggerindex_list.html'


# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-03-16 10:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('collector', '0005_loggerindex_durable'),
    ]

    operations = [
        migrations.AlterField(
            model_name='loggerindex',
            name='routing_key',
            field=models.TextField(default=b'logstash'),
        ),
    ]



# coding: utf-8

# $Id: $
from rest_framework import serializers


class JSONSerializerField(serializers.Field):
    """ Serializer for JSONField -- required to make field writable"""
    def to_internal_value(self, data):
        return data
    def to_representation(self, value):
        return value




# coding: utf-8

# $Id: $
from django.contrib import admin
from alco.collector.models import LoggerIndex, LoggerColumn


class LoggerIndexAdmin(admin.ModelAdmin):
    massadmin_exclude = ('name',)
    list_display = ('name', 'queue_name', 'routing_key')


class LoggerColumnAdmin(admin.ModelAdmin):
    massadmin_exclude = ('index', 'name',)
    list_display = ('name', 'index_name', 'filtered', 'display', 'context',
                    'excluded', 'indexed')
    list_filter = ('index',)

    def index_name(self, obj):
        return obj.index.name


admin.site.register(LoggerIndex, LoggerIndexAdmin)
admin.site.register(LoggerColumn, LoggerColumnAdmin)




# coding: utf-8

# $Id: $
from copy import copy
from rest_framework import serializers
from rest_framework.serializers import ModelSerializer
from sphinxsearch.models import SphinxDateTimeField
from rest_framework.fields import DateTimeField

from alco.grep.api.fields import JSONSerializerField


mapping = copy(ModelSerializer.serializer_field_mapping)
mapping[SphinxDateTimeField] = DateTimeField


class LogBaseSerializer(ModelSerializer):

    js = JSONSerializerField()
    datetime = serializers.SerializerMethodField()
    logline_snippet = serializers.SerializerMethodField()
    
    serializer_field_mapping = mapping

    def get_datetime(self, obj):
        return obj.datetime.isoformat()

    def get_logline_snippet(self, obj):
        return getattr(obj, 'logline_snippet', None)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='LoggerColumn',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', serialize=False, auto_created=True)),
                ('name', models.CharField(max_length=100)),
                ('filtered', models.BooleanField(default=False)),
            ],
        ),
        migrations.CreateModel(
            name='LoggerIndex',
            fields=[
                ('id', models.AutoField(primary_key=True, verbose_name='ID', serialize=False, auto_created=True)),
                ('name', models.CharField(max_length=20)),
                ('intervals', models.IntegerField(default=30)),
                ('queue_name', models.CharField(default='logstash', max_length=30)),
                ('routing_key', models.CharField(default='logstash', max_length=30)),
            ],
        ),
        migrations.AddField(
            model_name='loggercolumn',
            name='index',
            field=models.ForeignKey(to='collector.LoggerIndex'),
        ),
    ]



# coding: utf-8

# $Id: $
import time
from django.forms import fields
from django.utils.functional import cached_property
import django_filters
from rest_framework import filters as rf_filters
from rest_framework.fields import DateTimeField
from rest_framework.generics import ListAPIView

from alco.collector.models import LoggerIndex
from alco.grep.api import filters
from alco.grep.api.pagination import CursorLogPaginator
from alco.grep.models import create_index_model
from alco.grep.api.serializers import LogBaseSerializer


class TimestampPKField(fields.DateTimeField):
    def to_python(self, value):
        dt = super(TimestampPKField, self).to_python(value)
        if dt is None:
            return None
        return time.mktime(dt.timetuple()) * 10 ** 9


class TimestampPKFilter(django_filters.DateTimeFilter):
    field_class = TimestampPKField


class GrepView(ListAPIView):
    filter_backends = (rf_filters.DjangoFilterBackend,
                       filters.SphinxSearchFilter,
                       filters.JSONFieldFilter)

    pagination_class = CursorLogPaginator

    @property
    def filter_class(self):
        class TimestampFilter(django_filters.FilterSet):
            start_ts = TimestampPKFilter(name="pk", lookup_type='gte')
            end_ts = TimestampPKFilter(name="pk", lookup_type='lte')

            class Meta:
                model = self.log_model
                fields = ['id']

        return TimestampFilter

    @cached_property
    def index(self):
        return LoggerIndex.objects.get(name=self.kwargs['logger'])

    @cached_property
    def log_model(self):
        start_ts = self.request.GET.get('start_ts') or ''
        try:
            f = DateTimeField()
            distr = f.to_internal_value(start_ts).strftime('%Y%m%d')
        except Exception:
            distr = None

        return create_index_model(self.index, distr=distr)

    def get_serializer_class(self):
        class Meta:
            model = self.log_model
            fields = ('id', 'datetime', 'logline', 'logline_snippet', 'js')

        serializer_class = type('LogSerializer', (LogBaseSerializer,),
                                {'Meta': Meta})

        return serializer_class

    def get_queryset(self):
        page = int(self.request.GET.get('page', 1)) + 1
        per_page = self.pagination_class.page_size
        return self.log_model.objects.options(max_matches=page * per_page)

    def get_json_fields(self, request):
        fields = set(self.index.visible_fields) - set(self.index.indexed_fields)
        result = []
        for key, value in request.GET.items():
            field = key.split('__')[0]
            if field in fields:
                result.append(field)
        return result

    def get_search_fields(self, request):
        fields = set(self.index.visible_fields) & set(self.index.indexed_fields)
        result = []
        for key, value in request.GET.items():
            field = key.split('__')[0]
            if field in fields:
                result.append(field)
        return result







# coding: utf-8

# $Id: $
import datetime
from dateutil import rrule
from django.db import models
from django.utils.functional import cached_property
from django.utils.timezone import localtime, now


class LoggerIndex(models.Model):

    name = models.CharField(max_length=20)
    intervals = models.IntegerField(default=30)
    queue_name = models.CharField(max_length=30, default='logstash')
    routing_key = models.TextField(default='logstash')
    durable = models.BooleanField(default=False, blank=True)
    num_processes = models.IntegerField(default=1)

    @property
    def index_names(self):
        return [d.strftime("%Y%m%d") for d in self.index_dates]

    @property
    def index_dates(self):
        today = localtime(now()).date()
        midnight = today + datetime.timedelta(days=1)
        start = midnight - datetime.timedelta(days=self.intervals)
        dates = rrule.rrule(
            rrule.DAILY,
            dtstart=start,
            until=midnight)
        return list(dates)

    @cached_property
    def field_names(self):
        return list(self.loggercolumn_set.values_list('name', flat=True))

    @cached_property
    def filtered_fields(self):
        return list(self.loggercolumn_set.filter(
            filtered=True).values_list('name', flat=True))

    @cached_property
    def visible_fields(self):
        return list(self.loggercolumn_set.filter(
            display=True).values_list('name', flat=True))

    @cached_property
    def indexed_fields(self):
        return list(self.loggercolumn_set.filter(
            indexed=True).values_list('name', flat=True))

    def __str__(self):
        return self.name


class LoggerColumn(models.Model):
    class Meta:
        unique_together = ('index', 'name')

    index = models.ForeignKey(LoggerIndex)
    name = models.CharField(max_length=100)
    filtered = models.BooleanField(default=False)
    display = models.BooleanField(default=True)
    excluded = models.BooleanField(default=False)
    context = models.BooleanField(default=False)
    indexed = models.BooleanField(default=False)

    def __str__(self):
        return '%s(%s)' % (self.name, self.index)


"""lcm URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/1.8/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  url(r'^$', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  url(r'^$', Home.as_view(), name='home')
Including another URLconf
    1. Add an import:  from blog import urls as blog_urls
    2. Add a URL to urlpatterns:  url(r'^blog/', include(blog_urls))
"""
from django.conf.urls import include, url
from django.contrib import admin

urlpatterns = [
    url(r'^admin/', include(admin.site.urls)),
    url(r'^admin/', include('massadmin.urls')),
    url(r'^grep/', include('alco.grep.urls')),
    url(r'^collector/', include('alco.collector.urls')),
    url(r'^api/grep/', include('alco.grep.api.urls')),
    url(r'^api/collector/', include('alco.collector.api.urls')),

]



# coding: utf-8

# $Id: $
from django.utils.timezone import now, localtime

from django.views.generic import ListView
from django.conf import settings
from alco.collector.defaults import SPHINX_CONFIG

from alco.collector.models import LoggerIndex


class LoggerIndexView(ListView):
    model = LoggerIndex
    content_type = 'text/plain'
    template_name = 'collector/sphinx.conf'

    def get_context_data(self, **kwargs):
        cd = super(LoggerIndexView, self).get_context_data(**kwargs)
        cd['settings'] = settings
        cd['config'] = SPHINX_CONFIG
        cd['now'] = localtime(now()).strftime('%Y-%m-%d %H:%M:%S')
        return cd


# coding: utf-8

# $Id: $

from django.conf import settings

__all__ = ['ALCO_SETTINGS']

# default settings for alco package
ALCO_SETTINGS = {
    # log messaging server
    'RABBITMQ': {
        'host': '127.0.0.1',
        'userid': 'guest',
        'password': 'guest',
        'virtual_host': '/',
    },

    # redis for generating primary key sequences
    'REDIS': {
        'host': '127.0.0.1',
        'db': 0
    },
    # url for fetching sphinx.conf dynamically
    'SPHINX_CONF_URL': 'http://127.0.0.1:8000/',
    # name of django.db.connection for SphinxSearch
    'SPHINX_DATABASE_NAME': 'sphinx',
    # number of results in log view API
    'LOG_PAGE_SIZE': 100,
    'RABBITMQ_API_PORT': 55672
}

# Allow to override some settings from project django conf.

_custom = getattr(settings, 'ALCO_SETTINGS', {})
for k, v in _custom.items():
    ALCO_SETTINGS[k] = v

SPHINX_CONFIG = {
    'index': {
        'morphology': 'none',
        'min_word_len': 4,
    },
    'searchd': {
        'listen': '9306:mysql41',
        'log': '/var/log/sphinxsearch/searchd.log',
        'query_log': '/var/log/sphinxsearch/query.log',
        'read_timeout': 5,
        'client_timeout': 300,
        'max_children': 30,
        'dist_threads': 4,
        'ondisk_attrs_default': 'pool',
        'persistent_connections_limit': 30,
        'pid_file': '/var/run/sphinxsearch/searchd.pid',
        'seamless_rotate': 1,
        'preopen_indexes': 1,
        'unlink_old': 1,
        'binlog_path': '',  # disable logging
    }
}

# Allow to override sphinx.conf variables from project django conf.

_custom = getattr(settings, 'ALCO_SPHINX_CONF', {})
for section in SPHINX_CONFIG.keys():
    for k, v in _custom.get(section, {}).items():
        SPHINX_CONFIG[section][k] = v


from setuptools import setup

try:
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read_md = lambda f: open(f, 'r').read()

setup(
    name='alco',
    version='2.1.0',
    long_description=read_md('README.md'),
    packages=['alco',
              'alco.grep',
              'alco.grep.api',
              'alco.grep.migrations',
              'alco.grep.templatetags',
              'alco.collector',
              'alco.collector.api',
              'alco.collector.management',
              'alco.collector.management.commands',
              'alco.collector.migrations',
    ],
    url='https://github.com/rutube/alco',
    include_package_data=True,
    package_data={
        'alco': [
            'alco/collector/templates/',
            'alco/grep/templates/',
            'alco/grep/static/',
        ],
    },
    install_requires=[
        'Django>=1.8',
        'djangorestframework>=3.1',
        'dateutils',
        'django-filter>=0.10.0',
        'django-mass-edit',
        'jsonfield',
        'python-daemon',
        'redis',
        'pyrabbit',
        'django_sphinxsearch',
        'ujson',
        'librabbitmq',
        'logutils',
    ],
    license='Beerware',
    author='tumbler',
    author_email='zimbler@gmail.com',
    description='Autonomous Log Collector and Observer'
)



# coding: utf-8

# $Id: $
import time
from datetime import datetime
from django.db import models
from jsonfield import JSONField

from sphinxsearch.models import SphinxModel, SphinxField
from alco.collector.models import LoggerIndex, LoggerColumn


class Shortcut(models.Model):
    name = models.CharField(max_length=30, primary_key=True)
    description = models.CharField(max_length=255, blank=True, default='')
    url = models.CharField(max_length=255)
    index = models.ForeignKey(LoggerIndex)
    default_field = models.ForeignKey(LoggerColumn, blank=True, null=True)

    def __str__(self):
        return self.name


class LogBase(SphinxModel):
    class Meta:
        abstract = True
        ordering = ('pk',)
        db_table = 'logger'

    logline = models.TextField()
    js = JSONField()

    @property
    def ts(self):
        return self.pk / (10**9)

    @property
    def ms(self):
        return (self.pk / 1000) % 10**6

    @property
    def datetime(self):
        return datetime.fromtimestamp(self.timestamp)

    @property
    def timestamp(self):
        return (self.pk / 1000) / 1000000.0


def create_char_field(c):
    return models.CharField(max_length=255, db_column='js.%s' % c)


def create_sphinx_field(c):
    return SphinxField(max_length=255, db_column=c)


def create_index_model(index, distr=None):
    model_name = str("%sLog" % index.name.title())
    if distr:
        table = str('%s_%s_distr' % (index.name, distr))
    else:
        table = str(index.name)

    class Meta:
        db_table = table

    fields = {}

    for column in index.loggercolumn_set.all():
        if column.name in ('pk', 'id'):
            continue
        if column.indexed:
            fields[column.name] = create_sphinx_field(column.name)
        else:
            fields[column.name] = create_char_field(column.name)

    fields.update({'Meta': Meta, '__module__': __name__})

    Log = type(model_name, (LogBase,), fields)
    return Log





# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations


class Migration(migrations.Migration):

    dependencies = [
        ('collector', '0003_loggercolumn_context'),
    ]

    operations = [
        migrations.CreateModel(
            name='Shortcut',
            fields=[
                ('name', models.CharField(max_length=30, serialize=False, primary_key=True)),
                ('description', models.CharField(blank=True, max_length=255, default='')),
                ('url', models.CharField(max_length=255)),
                ('default_field', models.ForeignKey(to='collector.LoggerColumn', null=True)),
                ('index', models.ForeignKey(to='collector.LoggerIndex')),
            ],
        ),
    ]



# coding: utf-8

# $Id: $
from rest_framework import fields
from rest_framework.fields import ListField
from rest_framework.serializers import ModelSerializer
from alco.collector.models import LoggerIndex


class LoggerIndexSerializer(ModelSerializer):
    class Meta:
        model = LoggerIndex

    index_names = ListField()